from flask import Flask, request, jsonify
import random
import time

app = Flask(__name__)
port = 3000


class RobotCommands:
    tasks = []
    tasksIndex = 0

    def get_random_int(self, min=0, max=1000):
        return random.randint(min, max)

    def get_random_command(self):
        return random.choice(commands)

    def create_instructions(self, commands: list[str]):
        global tasks
        instructions = {"LuckyCode": []}
        batch_id = self.get_random_int()
        for command in commands:
            instructions["LuckyCode"].append({
                "ID": str(self.get_random_int()),
                "batchId": batch_id,
                "code": command,
                "time": str(int(time.time())),
                "callback": "on"
            })
        instructions["status"] = "queued"
        robot_commands.tasks.append(instructions)

    def check_if_batch_is_complete(self, task_id):
        batch_id = None
        for taskArray in robot_commands.tasks:
            for task in taskArray["LuckyCode"]:
                if task["ID"] == task_id:
                    batch_id = task["batchId"]
                    print(f"Batch ID matching task ID {task_id} is {batch_id}")

        is_complete = False
        for taskArray in robot_commands.tasks:
            for task in taskArray["LuckyCode"]:
                if task["batchId"] == batch_id:
                    all_completed = all(t.get("status") == "completed" for t in taskArray["LuckyCode"])
                    if all_completed:
                        print(f"Batch {batch_id} is complete")
                        is_complete = True
        return is_complete

    def mark_task_as_complete(self, task_id):
        for taskArray in robot_commands.tasks:
            for task in taskArray["LuckyCode"]:
                if task["ID"] == task_id:
                    task["status"] = "completed"

robot_commands = RobotCommands()

@app.route('/', methods=['POST'])
def handle_post():
    body = request.get_json()
    print('Received POST request with payload:', body)

    if "ID" in body:
        print(f"{body['ID']} is finished")
        robot_commands.mark_task_as_complete(body["ID"])
        if robot_commands.check_if_batch_is_complete(body["ID"]):
            if robot_commands.tasksIndex < len(robot_commands.tasks) - 1:
                print("batch is complete increasing index")
                robot_commands.tasksIndex += 1
            else:
                print("all tasks complete waiting for new ones...")
    return 'POST request received'


@app.route('/', methods=['GET'])
def handle_get():
    next_command = robot_commands.tasks[robot_commands.tasksIndex]
    if next_command["status"] == "queued":
        # print("sending next_command", next_command)
        next_command["status"] = "in-progress"
        return jsonify(next_command)
    else:
        return jsonify({"LuckyCode": []})


if __name__ == '__main__':
    commands = [["RESET"], ["w 5650 1", "a 30 1"], ["A 0 1", "W 18000 1"], ["w 2500 1", "d 30 1", "EX1 10", "EX2 10", "G 100 1"], ["w 3000 1", "a 0 1", "u 100"], ["u -200"]]
    for command in commands:
        robot_commands.create_instructions(command)
    print("tasks", robot_commands.tasks)
    app.run(port=port)


from flask import Flask, request, jsonify
import random
import time

app = Flask(__name__)
port = 3000
tasks = []
tasksIndex = 0

def getRandomInt(min=0, max=1000):
    return random.randint(min, max)

def createInstructions(commands=[]):
    instructions = {"LuckyCode": []}
    batchID = getRandomInt()
    for command in commands:
        instructions["LuckyCode"].append({
            "ID": str(getRandomInt()),
            "batchID": batchID,
            "code": command,
            "time": str(int(time.time())),
            "callback": "on"
        })
    instructions["status"] = "queued"
    tasks.append(instructions)

def checkIfBatchIsComplete(taskID):
    batchID = None
    for taskArray in tasks:
        for task in taskArray["LuckyCode"]:
            if task["ID"] == taskID:
                batchID = task["batchID"]
                print(f"Batch ID matching task ID {taskID} is {batchID}")

    isComplete = False
    for taskArray in tasks:
        for task in taskArray["LuckyCode"]:
            if task["batchID"] == batchID:
                allCompleted = all(t["status"] == "completed" for t in taskArray["LuckyCode"])
                if allCompleted:
                    print(f"Batch {batchID} is complete")
                    isComplete = True
    return isComplete

def markTaskAsComplete(taskID):
    for taskArray in tasks:
        for task in taskArray["LuckyCode"]:
            if task["ID"] == taskID:
                task["status"] = "completed"

@app.route('/', methods=['POST'])
def handle_post():
    body = request.get_json()
    print('Received POST request with payload:', body)

    if "ID" in body:
        print(f"{body['ID']} is finished")
        markTaskAsComplete(body["ID"])

        if checkIfBatchIsComplete(body["ID"]):
            global tasksIndex
            if tasksIndex < len(tasks) - 1:
                print("batch is complete increasing index")
                tasksIndex += 1
            else:
                print("all tasks complete waiting for new ones...")
    return 'POST request received', 200

@app.route('/', methods=['GET'])
def handle_get():
    global tasksIndex
    nextCommand = tasks[tasksIndex]
    if nextCommand["status"] == "queued":
        # print("sending nextCommand", nextCommand)
        nextCommand["status"] = "in-progress"
        return jsonify(nextCommand)
    else:
        return jsonify({"LuckyCode": []})

if __name__ == '__main__':
    commands = [["RESET"], ["w 5650 1", "a 30 1"], ["A 0 1", "W 18000 1"], ["w 2500 1", "d 30 1", "EX1 10", "EX2 10", "G 100 1"], ["w 3000 1", "a 0 1", "u 100"], ["u -200"]]
    for command in commands:
        createInstructions(command)
    print("tasks", tasks)
    app.run(port=port)


from flask import Flask, Response
import cv2
import numpy as np
import base64

app = Flask(__name__)

@app.route('/showImage')
def show_image():
    # Define a base64 encoded image string
    base64_image = open('./base64img.txt', 'r').read().strip()

    # Decode the base64 string
    image_data = base64.b64decode(base64_image)

    # Convert to numpy array
    np_arr = np.frombuffer(image_data, np.uint8)

    # Decode the image
    image = cv2.imdecode(np_arr, cv2.IMREAD_COLOR)

    # Encode image to send as HTTP response
    _, buffer = cv2.imencode('.jpg', image)
    response = Response(buffer.tobytes(), mimetype='image/jpeg')
    return response

if __name__ == '__main__':
    app.run(debug=True)



import asyncio
import cv2
import numpy as np
from aiortc import RTCPeerConnection, VideoStreamTrack
from aiortc.contrib.media import MediaBlackhole, MediaPlayer, MediaRecorder

class VideoImageTrack(VideoStreamTrack):
    """
    We're not going to use this anymore - so let's delete soon.
    A video track that returns an image received from the WebRTC stream.
    """
    def __init__(self, track):
        super().__init__()  # don't forget this!
        self.track = track

    async def recv(self):
        frame = await self.track.recv()

        # convert PIL image to OpenCV format
        img = cv2.cvtColor(np.array(frame.to_pil()), cv2.COLOR_RGB2BGR)

        # process image with OpenCV
        img = cv2.Canny(img, 100, 200)

        # return modified image
        return img

async def run(pc):
    def add_tracks():
        if player and player.video:
            pc.addTrack(VideoImageTrack(player.video))

    @pc.on("track")
    def on_track(track):
        print("Track %s received" % track.kind)
        if track.kind == "video":
            local_video = VideoImageTrack(track)
            pc.addTrack(local_video)

    player = MediaPlayer('ws://localhost:8888')
    add_tracks()

    await pc.setLocalDescription(await pc.createOffer())
    await asyncio.sleep(1)

if __name__ == "__main__":
    pc = RTCPeerConnection()
    loop = asyncio.get_event_loop()
    try:
        loop.run_until_complete(run(pc))
    except Exception as e:
        print(e)
    finally:
        loop.run_until_complete(pc.close())


import paho.mqtt.client as mqtt
import time
import cv2
import numpy as np
import base64
import queue
import threading
# Define the MQTT broker details
broker = "0.0.0.0"  # Example broker, replace with your broker
port = 1883
image_topic = "test/image"
text_topic = "test/text"
task_complete = "task/complete"

i=0

with open('./base64img.txt', 'r') as file:
    base64_image = file.read().strip()

# Create a new MQTT client instance
client = mqtt.Client()

# Connect to the broker
client.connect(broker, port, 60)

def show_image(base64_image):
    image_data = base64.b64decode(base64_image)
    # Save image_data as file
    with open(f'./data/{i}.png', 'wb') as file:
        file.write(image_data)
    # Convert to numpy array
    np_arr = np.frombuffer(image_data, np.uint8)

    # Decode the image
    image = cv2.imdecode(np_arr, cv2.IMREAD_COLOR)
    global i
    cv2.imwrite( f'./data/decoded_image_{i}.jpg', image)
    i+=1
    # cv2.imshow('Decoded Image', image)
    # cv2.waitKey(1)
    #cv2.destroyAllWindows()

# Define the callback function for when a message is received
def on_message(client, userdata, msg):
    if msg.topic == image_topic:
        img= msg.payload.decode()
        show_image(img)
        

    elif msg.topic == text_topic:
        text = msg.payload.decode()
        print(f"Received text: {text}")

# Set the on_message callback function
client.on_message = on_message

# Subscribe to the topics
client.subscribe(image_topic)
client.subscribe(text_topic)
client.subscribe(task_complete)
#publish test message


# Start the MQTT client loop to process network traffic and dispatch callbacks
client.loop_start()

try:
    while True:

        # client.publish(text_topic, "Terminalden selaminaleykum!")
        client.publish(image_topic, base64_image)
        time.sleep(2)
except KeyboardInterrupt:
    print("Disconnecting from broker...")
    client.loop_stop()
    client.disconnect()
    cv2.destroyAllWindows()



# 0. Folder monitor for new files - watchdog - done
# 0a. Use emitter https://pyee.readthedocs.io/en/latest/api/
# 1. Combine images by number, name and extension
# 2. Create video stream from the images chunks
# 3. Use opencv to preview the realtime stream
# 4. Feed stream to yolo
# 5. Create library lucky robots installable via pip


import luckyrobots.src as lr
from luckyrobots.src.events import on_message

lr.binary_path = "C:\\Users\\Goran\\Downloads\\Windows06_28_2024\\Windows06_28_2024"

@on_message("robot_images_created")
def handle_file_created(robot_images: list):
    print(f"Images created: {len(robot_images)}")

lr.start()



from . import event_emitter


class ProcessImage:
    def __init__(self):
        pass

    def process(self, image_bytes, image_name):
        if "head_cam" in image_name:
            print("Processing head_cam image")
            event_emitter.emit("head_cam", image_bytes, image_name)
        elif "hand_cam" in image_name:
            print("Processing hand_cam image")
            event_emitter.emit("hand_cam", image_bytes, image_name)
        elif "body_pos" in image_name:
            print("Processing body_pos image")
            event_emitter.emit("body_pos", image_bytes, image_name)
        elif "rgb_cam1" in image_name:
            print("Processing rgb_cam1 image")
            event_emitter.emit("rgb_cam1", image_bytes, image_name)
        elif "rgb_cam2" in image_name:
            print("Processing rgb_cam2 image")
            event_emitter.emit("rgb_cam2", image_bytes, image_name)
        elif "depth_cam1" in image_name:
            print("Processing depth_cam1 image")
            event_emitter.emit("depth_cam1", image_bytes, image_name)
        elif "depth_cam2" in image_name:
            print("Processing depth_cam2 image")
            event_emitter.emit("depth_cam2", image_bytes, image_name)
        return image_bytes



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="luckyrobots",
    version="0.1.0",
    description="Robotics-AI Training in Hyperrealistic Game Environments",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Devrim Yashar",
    author_email="dvrmysr@gmail.com",
    url="https://github.com/lucky-robots/lucky-robots",
    packages=find_packages(),
    install_requires=[
        "watchdog", "pyee", "Flask==2.3.2"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)